#version 150
precision highp float;

#define CALC_MIDDLE_GRAY 0
#define CALC_ADAPTIVE_LUM 1

const vec3 LUM_CONVERT = vec3(0.299f, 0.587f, 0.114f);
const float Tau = 0.5f;


uniform sampler2D lastAdaptedLumTexture;
uniform sampler2D lumTexture; //luminance of the curent scene (middle gray)
uniform int operationMode;
uniform float frameTime;
uniform int mipmapLevel;
//uniform float sampleSizeX;
//uniform float sampleSizeY;
uniform vec2 sampleSize;

//in vec2 colorTexUV;

out vec4 fragColor;

void main()
{
	if(operationMode == CALC_MIDDLE_GRAY)
	{
		fragColor = texture2D(lumTexture, vec2(0.5f, 0.5f));
	}
	else // CALC_ADAPTIVE_LUM
	{
		float lastAdaptedLum = texture2D(lastAdaptedLumTexture, vec2(0.5f, 0.5f)).a;
		
		// Calculate the avarage lum of the current frame
		vec3 curFrame =  textureLod(lumTexture, vec2(0.5f, 0.5f), mipmapLevel).rgb;
		
		// nur wegen extrem hellen stellen in godRays
		/*
		float minColor = 0.2;
		if(curFrame.r < minColor) {curFrame.r = minColor;}
		if(curFrame.g < minColor) {curFrame.g = minColor;}
		if(curFrame.b < minColor) {curFrame.b = minColor;}
		float maxColor = 1;
		if(curFrame.r > maxColor) {curFrame.r = maxColor;}
		if(curFrame.g > maxColor) {curFrame.g = maxColor;}
		if(curFrame.b > maxColor) {curFrame.b = maxColor;}

		float avg = 0.3;
		curFrame.r = avg;
		curFrame.g = avg;
		curFrame.b = avg;*/

		float curLum = dot(curFrame, LUM_CONVERT);


		// Adapt the luminance using Pattanaik's technique
		float adaptedLum = lastAdaptedLum + (curLum - lastAdaptedLum) * (1 - exp(-frameTime * Tau));

		fragColor = vec4(curFrame.r, curFrame.g, curFrame.b, adaptedLum);
	}
}