#version 410
precision highp float;

//#define COLOR_TEST 1

const float PI = 3.141592653589;
const float logScale = 10.0;

uniform bool wireframe = false;
uniform sampler2D depthTexture;
uniform mat4 inverseCameraMatrix;
uniform vec4 cameraWorldPos;
uniform vec3 lightWorldPos;

uniform vec3 lightIntensity;
uniform float beta;
uniform float maxAValue; // scalierungsfactor fr text loockup val
uniform float maxBValue;
uniform float intMinAB; // minimale und maximale integralsgrenzen
uniform float intMaxAB;
uniform float intMinC;
uniform float intMaxC;
uniform sampler2D loockupTableA;
uniform sampler2D loockupTableB;

in vec4 eWorlPos;
#ifdef COLOR_TEST
	in vec3 eTestColor;
#endif

out vec4 fragColor;

float mapTexCoordAB(float yAB)
{
	float texCoordAB = (abs(yAB) - intMinAB) / (intMaxAB - intMinAB);
	//return texCoordAB;
	float logCoord = ((log(texCoordAB) + logScale) / logScale);
	//if(isinf(logCoord))
	if(logCoord < 0.0)
		logCoord = 0.0;
	return sign(yAB) * logCoord / 2.0 + 0.5;
}

float mapTexCoordC(float yC)
{
	float texCoordC = (abs(yC) - intMinC) / (intMaxC - intMinC);
	//return texCoordC;
	float logCoord = (log(texCoordC) + logScale) / logScale;
	//if(isinf(logCoord))
	if(logCoord < 0.0)
		logCoord = 0.0;
	return logCoord;
}

void main()
{
	if(!wireframe)
	{
fragColor = vec4(0.0, 0.0, 0.0, 1.0);
	// Divide by w to get the world position.
	//vec4 worldPos = eWorlPos / eWorlPos.w;
	vec3 worldPos = eWorlPos.xyz;

	//world Pos der Opacen Geometrie (aus dem deferred render step) zurck rechnen
	float depth = texelFetch(depthTexture, ivec2(gl_FragCoord.xy), 0 ).r;
	vec2 fragCoord = gl_FragCoord.xy / textureSize(depthTexture, 0);
	vec3 coord = 2.0 * vec3( fragCoord, depth ) - vec3(1.0);
	vec4 worldGeometry = inverseCameraMatrix * vec4( coord, 1.0 );
	worldGeometry /= worldGeometry.w;

	// ohne z-Test rendern	.... glDisable(GL_DEPTH_TEST);
	float cam2lightVolumeDist = length(worldPos - cameraWorldPos.xyz);
	float cam2geometryDist = length(worldGeometry.xyz - cameraWorldPos.xyz);
	if(cam2geometryDist < cam2lightVolumeDist)
	{
		worldPos = worldGeometry.xyz;
	}

	vec3 viewVec = worldPos.xyz - cameraWorldPos.xyz; // vom kamera mitelpunkt zum aktuellen geometrie punkt //z
	vec3 viewDir = normalize(viewVec);
	float viewDist = length(viewVec);

	vec3 lightVec = lightWorldPos - cameraWorldPos.xyz; // von der kamera zum Licht
	
	float lightViePro = dot(lightVec, viewDir); // lightPosFromCamera auf viewVec projezieren
	/*
	vec3 a = viewDir * lightViePro;  // viewDir mal der lnge der Projektion von lightPosFromCamera auf viewVec
	vec3 b = viewVec - a; //von a nach fragment dist
	vec3 c = lightVec - a; // von a zur licht pos

	//float aDist = length(a);
	//float bDist = length(b);
	//float cDist = length(c);
	*/
	float aDist = lightViePro; // negertiv wen das licht hinter der camera ist
	float bDist = viewDist - aDist; // negertiv wen licht hinter der fragment pos ist
	float cDist;
	float cDistPow2 = dot(lightVec,lightVec) - aDist*aDist; 
	if(cDistPow2 <= 0.1) // 0.1 bast fr ein beta von 0.04 
		cDist = 0.316227766016837933199889354; //sqrt(0.1); // ganz ganz selten aber es pasiert und dan hauts den tonemapper fr immer auf die schnauze...
		// das bedeutet bei einer textur die 512 pixel hoch ist das alle pixel unter 372 ignoriert werden....... (	((log(sqrt(0.1) / 4.8) + 10) / 10) = 0.7280092...	)
	else
		cDist =  sqrt(cDistPow2);
	
	float aStrich = beta * aDist;
	float bStrich = beta * bDist;
	float cStrich = beta * cDist;

	float texCoordA = mapTexCoordAB(aStrich);
	float texCoordB = mapTexCoordAB(-bStrich);
	float texCoordC = mapTexCoordC(cStrich);

	
	//fragColor = vec4(texCoordA * 0.5, texCoordB * 0.5, texCoordC * 0.5, 1.0);
	//fragColor = vec4(abs(aDist) * 1.0/180.0, abs(bDist) * 0.0, abs(cDist) * 0.0, 1.0);
	/*if(aDist < 0)
	{
		fragColor = vec4(1.0, 0.0, 0.0, 1.0);
	}
	else
	{
		fragColor = vec4(0.0, 1.0, 0.0, 1.0);
	}
	*/

	/* bitte nicht lschen !!!!!!!!!!!
	// versuche die frag post zu visualisieren
	vec3 lightWorldPosOnGround = lightWorldPos + vec3(0.0, -8.0, 0.0);
	vec3 fragPostFromGroundCenter = worldPos.xyz - lightWorldPosOnGround;
	fragColor = vec4(fragPostFromGroundCenter / 8.0, 1.0);
	// => lightWorldPosStimmt stimmt, worlPos stimmt!
	// was ist mit der camera position?
	//fragColor = vec4(cameraWorldPos.xyz, 1.0); // fehler gefunden
	*/

	
	// a/b => x, c => y
	float lookupValA =  texture(loockupTableA, vec2(texCoordA, texCoordC)).r;
	float lookupValB =  texture(loockupTableB, vec2(texCoordB, texCoordC)).r;

	float airlightA = lookupValA * maxAValue;
	float airlightB = lookupValB * maxBValue;
	//airlightA = exp(airlightA);
	//airlightB = exp(airlightB);

	float ae = 1, be = 1;
	if( aDist > 0 && bDist < 0 ) // wen licht hinter dem fragment liegt
	{
		//be = exp( -beta * viewDist ); // quasi e^-zStrich ???
		be = exp(-aStrich);
		fragColor.r = 1.0;
	}
	else if( aDist > 0 && bDist > 0 ) // licht zwichen camera und fragmentPos
	{
		//be = exp( -aStrich );
		fragColor.g = 1.0;
	}
	else if( aDist < 0 && bDist > 0 ) // licht hinter der camera
	{
		//ae = be = exp( -aStrich );
		// a mega negertive
		// abs(a) > b
		// => a runter scalieren oder b rauf .....
		
		//be = exp( -aStrich ); // weiser fleck
		//ae = be = exp( -beta * viewDist   );
		//be = exp( beta * viewDist );
		//ae = exp( -bStrich );
		//ae = exp(-bStrich);
		//be = exp( -aStrich );
		fragColor.b = 1.0;
	}
	else // (aDist < 0 && bDist == 0) || (aDist > 0 && bDist < 0) // die warscheinlichkeit sollte in der gegend von unendlich klein sein...
	{
		//ae = be = -1000000;
	}
	be = exp(-aStrich);
	//ae = exp(-bStrich);
	

	float attenuation = ((beta * beta) / (4.0 * PI)) // sollten 4 * PI sein
						* (
							sign(aDist) * airlightA * ae 
							+ sign(bDist) * airlightB * be //be; exp(-aStrich)); //exp(-1 * aStrich));
						  );
	vec3 airlight = lightIntensity * attenuation / cStrich * 15.0; // mal 20 ist nur damits heller wird und 20 schaut einfach cool aus...

	//airlight = max(airlight, vec3(0.0, 0.0, 0.0));
	
	if(gl_FrontFacing)
	{
		// aus dem licht raus!!!!
		// in den shatten rein.
		//fragColor = vec4(0.0, 0.0, 1.0, 1.0);

		fragColor = vec4(airlight, 1.0);
	}
	else
	{
		// in das licht rein!!!!
		// aus dem schatten raus.
		//fragColor = vec4(1.0, 0.0, 0.0, 1.0);

		fragColor = vec4(-1 * airlight , 1.0);
	}
	
	/*
	if(abs(cDist) < 0.05)
	{
		fragColor = vec4(1.0, 0.0, 0.0, 1.0);
	}
	else
	{
		//fragColor = vec4(0.0, 0.0, abs(cDist), 1.0);
	}*/
	/*
	if(aDist < 0.0)
	{
		fragColor = vec4(1.0, 0.0, 0.0, 1.0);
	}
	else
	{
		fragColor = vec4(0.0, 0.0, 1.0, 1.0);
	}*/

	
	//fragColor = vec4(aDist/intMaxAB, 0.0, bDist/intMaxAB, 1.0);
	//fragColor = vec4(0.0, 0.0, bDist/intMaxAB, 1.0);
	//fragColor = vec4(0.0, 0.0, aDist/intMaxAB, 1.0);
	//fragColor += vec4(0.0, 0.0, cDist/intMaxC, 1.0);
	//fragColor = vec4(worldPos, 1.0);
	/*
	if(airlightA * ae < airlightB * be)
	{
		fragColor = vec4(1.0, 0.0, 0.0, 1.0);
	}
	else
	{
		fragColor = vec4(0.0, 0.0, 1.0, 1.0);
	}
	*/
	}
	else
	{
		#ifdef COLOR_TEST
			fragColor = vec4(eTestColor, 1.0);
		#else
			fragColor = vec4(0.0, 1.0, 0.0, 1.0);
		#endif
	}
}