#version 410
//precision highp float;

#define MAX_DIRECTIONAL_LIGHTS 2
#define MAX_POINT_LIGHTS 8
#define MAX_SHADOW_CASTING_POINT_LIGHTS 2
#define MAX_SPOT_LIGHTS 4
#define MAX_SHADOW_CASTIN_SPOT_LIGHTS 4

const float g_MinVariance = 0.000002; //0.00002;

struct DirectionalLight
{
	bool specularOn;
	vec3 inverseDir; // zeigt zum licht hin
	vec3 color;
};

struct PointLight
{
	bool specularOn;
	vec3 pos;
	vec3 color;
	
	bool attenuationOn;
	vec3 attenuation; // X=const, Y=linear, Z=quadratisch { 1/(X + Y*d + Z*d^2) }
	float maxDistanceOfInfluence;

	bool shadow;
	vec2 nearAndFarPlane; // near and far plane for cm-cams
	//samplerCube shadowCubeMap; // funktioniert leider nicht, drfte in bug sein => eigenes array
};

struct SpotLight
{
	bool specularOn;
	vec3 pos;
	vec3 color;

	bool attenuationOn;
	vec3 attenuation; // X=const, Y=linear, Z=quadratisch { 1/(X + Y*d + Z*d^2) }
	float maxDistanceOfInfluence;

	vec3 dir;
	float cosAngle;
	float radialAttenuationEpx;

	bool shadow;
	vec2 nearAndFarPlane; // near and far plane for light cam
	mat4 cameraMatrix;
	//sampler2D shadowMap;

	bool scattering;
};

uniform sampler2D depthTexture;
uniform sampler2D diffuseTexture;
uniform sampler2D specularTexture;
uniform sampler2D normalTexture;
uniform sampler2D emitTexture;
uniform float opticalThickness;

uniform vec4 cameraWorldPos;
uniform mat4 inverseCameraMatrix;
uniform DirectionalLight directionalLight[MAX_DIRECTIONAL_LIGHTS];
uniform int directionalLightCount = 0;
uniform PointLight pointLight[MAX_POINT_LIGHTS];
uniform int pointLightCount = 0;
uniform PointLight scPointLight[MAX_SHADOW_CASTING_POINT_LIGHTS];
//uniform samplerCube scPointLightShadowCubeMap[SHADOW_CASTING_POINT_LIGHTS];
uniform samplerCubeShadow scPointLightShadowCubeMap_0;
uniform samplerCubeShadow scPointLightShadowCubeMap_1;
//uniform samplerCubeShadow scPointLightShadowCubeMap_2;
//uniform samplerCubeShadow scPointLightShadowCubeMap_3;
uniform int scPointLightCount = 0;
uniform SpotLight spotLight[MAX_SPOT_LIGHTS];
uniform int spotLightCount = 0;
uniform SpotLight scSpotLight[MAX_SHADOW_CASTIN_SPOT_LIGHTS];
uniform sampler2D scSpotLightShadowMap_0;
uniform sampler2D scSpotLightShadowMap_1;
uniform sampler2D scSpotLightShadowMap_2;
uniform sampler2D scSpotLightShadowMap_3;
//uniform sampler2D scSpotLightShadowMap_4;
uniform sampler2D scSpotLightMoment2Map_0;
uniform sampler2D scSpotLightMoment2Map_1;
uniform sampler2D scSpotLightMoment2Map_2;
uniform sampler2D scSpotLightMoment2Map_3;
//uniform sampler2D scSpotLightMoment2Map_4;
uniform int scSpotLightCount = 0;

in vec2 colorTexUV;

out vec4 fragColor;


vec3 worldPos(float yZoverW)
{
	// H is the viewport position at this pixel in the range -1 to 1.  
	vec4 H = vec4(colorTexUV.x * 2.f - 1.0f, colorTexUV.y * 2.0f - 1.0f,  yZoverW * 2.0 - 1.0, 1.0f);
	// Transform by the view-projection inverse.  
	vec4 D = inverseCameraMatrix * H;  
	// Divide by w to get the world position.  
	vec4 worldPos = D / D.w;

	return worldPos.xyz;
}

// GPU Gems 3 S165
// http://fabiensanglard.net/shadowmappingVSM/index.php
float linstep(float yMin, float yMax, float yV)
{
	return clamp((yV - yMin) / (yMax - yMin), 0.0, 1.0);
}
float reduceLightBleeding(float yP_max, float yAmount)
{
	// Remove the [0, Amount] tail and linearly rescale (Amount, 1].
	return linstep(yAmount, 1, yP_max);
}



float chebyshevUpperBound(vec2 yMoments, float yDepth)  
{  
	// One-tailed inequality valid if t > Moments.x  
	float p;
	if(yDepth <= yMoments.x)
		p = 1.0;
	else 
		p = 0.0; // fr t > m
	// Compute variance.
	float variance = yMoments.y - (yMoments.x * yMoments.x);
	variance = max(variance, g_MinVariance);
	// Compute probabilistic upper bound.
	float d = yDepth - yMoments.x;
	float p_max = variance / (variance + d*d);
	p_max = max(p, p_max);

	// try to reduce light bleeding
	//if(p_max < 0.2)
	//	p_max = 0.0;

	p_max = reduceLightBleeding(p_max, 0.1);
	return p_max;
	//return 1.0;
}

/*
bool pointLightInShadow(int yIndex, vec3 yLight2GPointVec, float yLightDist, float yLightFarPlane)
{
	// calculate vector from surface point to light position
	// (both positions are given in world space)
	vec3 cubMapLookupVec = yLight2GPointVec; // von der lichtquelle zum aktuellen geometrie punkt (WS_pos.xyz - l_pos.xyz)

	// read depth value from cubemap shadow map //textureCube()
	// float smallestDist2Light = texture(shadowCubeMap, cubMapLookupVec).r; //texture(pointLightShadowCubeMap, cubMapLookupVec).r; //texture(pointLightShadowCubeMap[i], cubMapLookupVec).r;
	float smallestDist2Light = 0.0; //float smallestDist2Light = 0.1;
	switch (yIndex)
	{
		case 0: smallestDist2Light = texture(scPointLightShadowCubeMap_0, cubMapLookupVec).r;
			break;
		case 1: smallestDist2Light = texture(scPointLightShadowCubeMap_1, cubMapLookupVec).r;
			break;
		case 2: smallestDist2Light = texture(scPointLightShadowCubeMap_2, cubMapLookupVec).r;
			break;
		case 3: smallestDist2Light = texture(scPointLightShadowCubeMap_3, cubMapLookupVec).r;
			break;
	}

	// WS "dist-to-lightsource" for current fragment
	float currFragmentDist2Light = yLightDist; //length(cubMapLookupVec);

	// undo previous [0;1]-mapping of "dist-to-lightsource"
	smallestDist2Light *= yLightFarPlane; //scPointLight[i].nearAndFarPlane.y;

	float eps = 0.15; // add a small offset (adjust as needed)
	if((smallestDist2Light + eps) < currFragmentDist2Light)
	{
		// ==> fragment lies in shadow
		return true;
	}
	else
	{
		return false;
	}
}
*/

float pointLightInShadow(int yIndex, vec3 yLight2GPointVec, float yLightDist, float yLightFarPlane)
{
	// calculate vector from surface point to light position
	// (both positions are given in world space)
	vec3 cubMapLookupVec = yLight2GPointVec; // von der lichtquelle zum aktuellen geometrie punkt (WS_pos.xyz - l_pos.xyz)

	float eps = 0.10 / yLightFarPlane; // add a small offset (adjust as needed) => 0.20 works good for front faces
	float lightDist4Lookup = yLightDist/yLightFarPlane - eps;
	float p = 0.0;
	switch (yIndex)
	{
		case 0: p = texture(scPointLightShadowCubeMap_0, vec4(cubMapLookupVec, lightDist4Lookup));
			break;
		case 1: p = texture(scPointLightShadowCubeMap_1, vec4(cubMapLookupVec, lightDist4Lookup));
			break;
		//case 2: p = texture(scPointLightShadowCubeMap_2, vec4(cubMapLookupVec, lightDist4Lookup));
		//	break;
		//case 3: p = texture(scPointLightShadowCubeMap_3, vec4(cubMapLookupVec, lightDist4Lookup));
		//	break;
	}

	float minShadow = 0.35;
	p = linstep(0.0, 1.0 - minShadow, p);
	return p;
}

float spotLightInShadow(int yIndex, vec3 yWorldPos, float yLightDist, float yLightFarPlane)
{ 
	vec4 lightCamPos = scSpotLight[yIndex].cameraMatrix * vec4(yWorldPos, 1.0);
	//vec2 uv = (vec2(lightCamPos.x / lightCamPos.w, lightCamPos.y / lightCamPos.w) + 1.0) * 0.5;
	vec2 uv = vec2(lightCamPos.x / lightCamPos.w, lightCamPos.y / lightCamPos.w); // verschiebung von [-1, 1] auf [0, 1] wird bereits in die camera matrix hinen coodiert (C++ method: ShadowMapCamera::calcViewMatrix())

	// read depth value from shadow map
	float smallestDist2Light = 0.0; //float smallestDist2Light = 0.1;
	float moment2 = 0.0;
	switch (yIndex)
	{
		case 0: smallestDist2Light = texture(scSpotLightShadowMap_0, uv).a; // .r ohne blur
				moment2 = texture(scSpotLightMoment2Map_0, uv).a;
			break;
		case 1: smallestDist2Light = texture(scSpotLightShadowMap_1, uv).a;
				moment2 = texture(scSpotLightMoment2Map_1, uv).a;
			break;
		case 2: smallestDist2Light = texture(scSpotLightShadowMap_2, uv).a;
				moment2 = texture(scSpotLightMoment2Map_2, uv).a;
			break;
		case 3: smallestDist2Light = texture(scSpotLightShadowMap_3, uv).a;
				moment2 = texture(scSpotLightMoment2Map_3, uv).a;
			break;
		/*case 4: smallestDist2Light = texture(scSpotLightShadowMap_4, uv).a;
				moment2 = texture(scSpotLightMoment2Map_4, uv).a;*/
			//break;
	}

	// WS "dist-to-lightsource" for current fragment
	float currFragmentDist2Light = yLightDist; //length(cubMapLookupVec);

	vec2 moments = vec2(smallestDist2Light, moment2);
	return chebyshevUpperBound(moments, currFragmentDist2Light / yLightFarPlane); //shadowContribution

	// undo previous [0;1]-mapping of "dist-to-lightsource"
	//smallestDist2Light *= yLightFarPlane; //scPointLight[i].nearAndFarPlane.y;
	//moment2 *= yLightFarPlane;
	
	//return 1.0;
	/*
	float eps = 0.15; // add a small offset (adjust as needed)
	if((smallestDist2Light + eps) < currFragmentDist2Light)
	{
		// ==> fragment lies in shadow
		return true;
	}
	else
	{
		return false;
	}*/
}

void main()
{
	// daten aus den Deferred Texturen holen (G-Buffer)
	float depthT = texture2D(depthTexture, colorTexUV).r;

	vec4 diffuseColorT = texture2D(diffuseTexture, colorTexUV);
	vec3 diffuseColor = vec3(diffuseColorT);
	float diffuseIntensity = diffuseColorT.a * 63.0;

	vec4 specularColorT = texture2D(specularTexture, colorTexUV);
	vec3 specularColor = vec3(specularColorT);
	float specularIntensity = specularColorT.a * 63.0;

	vec4 normalT = texture2D(normalTexture, colorTexUV);
	vec3 normal = vec3(normalT);
	float specularExp = normalT.a;

	float emit = texture2D(emitTexture, colorTexUV).r;

	//-----------------------------------------------------------
	if(length(normal) > 0.0)
	{
		// postionenen rechnen
		vec3 worldPos = worldPos(depthT);
		vec3 viewVec = cameraWorldPos.xyz - worldPos;
		vec3 viewDir = normalize(viewVec); // vom Aktuellen geometrie punkt zum kamera mitelpunkt
		float viewDist = length(viewVec);

		// summed light intensities
		vec3 dLightColor = vec3(0.0, 0.0, 0.0);
		vec3 sLightColor = vec3(0.0, 0.0, 0.0);

		// Directional Lights
		for(int i=0; i < directionalLightCount; i++)
		{
			dLightColor += max(dot(normal, directionalLight[i].inverseDir), 0.0) * directionalLight[i].color;

			if(directionalLight[i].specularOn)
			{
				vec3 halfwayVector = normalize(directionalLight[i].inverseDir + viewDir);
				sLightColor += pow(max(dot(normal, halfwayVector), 0.0), specularExp) * directionalLight[i].color;
			}
		}

		// Point Lights
		for(int i=0; i < pointLightCount; i++)
		{
			vec3 lightVec = pointLight[i].pos - worldPos; // vom Aktuellen geometrie punkt zur lichtquelle
			vec3 lightDir = normalize(lightVec);
			float lightDist; // distanz vom object punkt zum licht

			if(pointLight[i].attenuationOn || pointLight[i].shadow)
			{
				lightDist = length(lightVec); 
			}

			bool noInfluence;
			if((pointLight[i].attenuationOn && lightDist > pointLight[i].maxDistanceOfInfluence))
			{ noInfluence = true; }
			else
			{ noInfluence = false; }

			if(!noInfluence)
			{
				vec3 dCurLightColor = max(dot(normal, lightDir), 0.0) * pointLight[i].color;
				vec3 sCurLightColor = vec3(0.0, 0.0, 0.0);

				if(pointLight[i].specularOn)
				{
					vec3 halfwayVector = normalize(lightDir + viewDir);
					sCurLightColor = pow(max(dot(normal, halfwayVector), 0.0), specularExp) * pointLight[i].color;
				}

				if(pointLight[i].attenuationOn)
				{
					// abname mit der distanz (kugel) berechnen
					// X=const, Y=linear, Z=quadratisch { 1/(X + Y*d + Z*d^2) }
					float angattenIntensity = 1.0 / dot(pointLight[i].attenuation, vec3(1.0, lightDist, lightDist * lightDist));
					dCurLightColor *= angattenIntensity;
					sCurLightColor *= angattenIntensity;
				}

				dLightColor += dCurLightColor;
				sLightColor += sCurLightColor;
			}
		}

		// Shadow Casting Point Lights
		for(int i=0; i < scPointLightCount; i++)
		{
			vec3 lightVec = scPointLight[i].pos - worldPos; // vom Aktuellen geometrie punkt zur lichtquelle
			vec3 lightDir = normalize(lightVec);
			float lightDist; // distanz vom object punkt zum licht

			if(scPointLight[i].attenuationOn || scPointLight[i].shadow)
			{
				lightDist = length(lightVec); 
			}

			bool noInfluence = false;
			if( (scPointLight[i].attenuationOn && lightDist > scPointLight[i].maxDistanceOfInfluence) 
				//|| 
				//( scPointLight[i].shadow && pointLightInShadow(i, (lightVec * -1), lightDist, scPointLight[i].nearAndFarPlane.y) )
				//pointLightInShadow(i, (lightVec * -1), lightDist, scPointLight[i].nearAndFarPlane.y)
				)
			{ noInfluence = true; }
			else
			{ noInfluence = false; }

			if(!noInfluence)
			{
				vec3 dCurLightColor = max(dot(normal, lightDir), 0.0) * scPointLight[i].color;
				vec3 sCurLightColor = vec3(0.0, 0.0, 0.0);

				if(scPointLight[i].specularOn)
				{
					vec3 halfwayVector = normalize(lightDir + viewDir);
					sCurLightColor = pow(max(dot(normal, halfwayVector), 0.0), specularExp) * scPointLight[i].color;
				}

				if(scPointLight[i].attenuationOn)
				{
					// abname mit der distanz (kugel) berechnen
					// X=const, Y=linear, Z=quadratisch { 1/(X + Y*d + Z*d^2) }
					float angattenIntensity = 1.0 / dot(scPointLight[i].attenuation, vec3(1.0, lightDist, lightDist * lightDist));
					dCurLightColor *= angattenIntensity;
					sCurLightColor *= angattenIntensity;
				}

				float shadowAttenuation = pointLightInShadow(i, (lightVec * -1), lightDist, scPointLight[i].nearAndFarPlane.y);
				dCurLightColor *= shadowAttenuation;
				sCurLightColor *= shadowAttenuation;

				dLightColor += dCurLightColor;
				sLightColor += sCurLightColor;
			}
		}

		// Spot Lights
		for(int i=0; i < spotLightCount; i++)
		{
			vec3 lightVec = spotLight[i].pos - worldPos;
			vec3 lightDir = normalize(lightVec);

			vec3 vObj = lightDir * -1; // zeigt von der lichtquelle zum objektPunkt
			float cosAlpha = dot(vObj,  spotLight[i].dir); // cos des winkel zwischen licht direction und dem vektor der von der lichtquelle zum aktuellen objekt punkt zeigt
			if(cosAlpha > spotLight[i].cosAngle)
			{
				// abnahme mit der distanz (kugel) berechnen
				float angattenIntensity = 1.0;
				if(spotLight[i].attenuationOn)
				{
					float lightDist = length(lightVec); // distanz vom pbject punkt zum licht
					if(lightDist < spotLight[i].maxDistanceOfInfluence)
					{
						// X=const, Y=linear, Z=quadratisch { 1/(X + Y*d + Z*d^2) }
						angattenIntensity = 1.0 / dot(spotLight[i].attenuation, vec3(1.0, lightDist, lightDist * lightDist));
					}
					else
					{
						angattenIntensity = 0.0;
					}
				}

				// abnahme mit dem radius (winkel zwischen kegel mitelpunkt und aktueler objekt position) berechnen
				float rAngattenIntensity = pow(cosAlpha, spotLight[i].radialAttenuationEpx); // den exponenten sollte man definierbar machen... (radial Attenuation Epx)
				dLightColor += max(dot(normal, lightDir), 0.0) * spotLight[i].color * rAngattenIntensity * angattenIntensity;

				if(spotLight[i].specularOn)
				{
					vec3 halfwayVector = normalize(lightDir + viewDir);
					sLightColor += pow(max(dot(normal, halfwayVector), 0.0), specularExp) * spotLight[i].color * rAngattenIntensity * angattenIntensity;
				}
			}
		}

		// sadow casting Spot Lights
		for(int i=0; i < scSpotLightCount; i++)
		{
			vec3 lightVec = scSpotLight[i].pos - worldPos;
			vec3 lightDir = normalize(lightVec);
			float lightDist; // distanz vom object punkt zum licht

			vec3 vObj = lightDir * -1; // zeigt von der lichtquelle zum objektPunkt
			float cosAlpha = dot(vObj,  scSpotLight[i].dir); // cos des winkel zwischen licht direction und dem vektor der von der lichtquelle zum aktuellen objekt punkt zeigt
			if(cosAlpha > scSpotLight[i].cosAngle)
			{
				if(scSpotLight[i].attenuationOn || scSpotLight[i].shadow)
				{
					lightDist = length(lightVec); 
				}

				bool noInfluence = false;
				if( (scSpotLight[i].attenuationOn && lightDist > scSpotLight[i].maxDistanceOfInfluence) 
					//|| 
					//( scSpotLight[i].shadow && pointLightInShadow(i, (lightVec * -1), lightDist, scSpotLight[i].nearAndFarPlane.y) )
					//spotLightInShadow(i, worldPos, lightDist, scSpotLight[i].nearAndFarPlane.y)
					)
				{ noInfluence = true; }
				else
				{ noInfluence = false; }

				if(!noInfluence)
				{
					// abnahme mit dem radius (winkel zwischen kegel mitelpunkt und aktueler objekt position) berechnen
					float rAngattenIntensity = pow(cosAlpha, scSpotLight[i].radialAttenuationEpx); // den exponenten sollte man definierbar machen... (radial Attenuation Epx)
					vec3 dCurLightColor = max(dot(normal, lightDir), 0.0) * scSpotLight[i].color * rAngattenIntensity;
					vec3 sCurLightColor = vec3(0.0, 0.0, 0.0);

					if(scSpotLight[i].specularOn)
					{
						vec3 halfwayVector = normalize(lightDir + viewDir);
						sCurLightColor = pow(max(dot(normal, halfwayVector), 0.0), specularExp) * scSpotLight[i].color * rAngattenIntensity;
					}

					if(scSpotLight[i].attenuationOn)
					{
						// abname mit der distanz (kugel) berechnen
						// X=const, Y=linear, Z=quadratisch { 1/(X + Y*d + Z*d^2) }
						float angattenIntensity = 1.0 / dot(scSpotLight[i].attenuation, vec3(1.0, lightDist, lightDist * lightDist));
						if(scSpotLight[i].scattering)
						{
							angattenIntensity *= exp(-1 * opticalThickness * lightDist) * exp(-1 * opticalThickness * viewDist);
						}
						
						dCurLightColor *= angattenIntensity;
						sCurLightColor *= angattenIntensity;
					}

					float shadowAttenuation = spotLightInShadow(i, worldPos, lightDist, scSpotLight[i].nearAndFarPlane.y);
					dCurLightColor *= shadowAttenuation;
					sCurLightColor *= shadowAttenuation;
					//dCurLightColor = vec3(shadowAttenuation, 0.0, 0.0);
					//fragColor = vec4(shadowAttenuation, shadowAttenuation, shadowAttenuation, 1.0);
					//fragColor = vec4(1.0, 1.0, 1.0, 1.0);

					dLightColor += dCurLightColor;
					sLightColor += sCurLightColor;
				}
			}

			/*
			vec4 lightCamPos = scSpotLight[0].cameraMatrix * vec4(worldPos, 1.0);
			vec2 uv = (vec2(lightCamPos.x / lightCamPos.w, lightCamPos.y / lightCamPos.w) + 1.0) * 0.5;
			// read depth value from shadow map
			float smallestDist2Light = texture(scSpotLightShadowMap_0, uv).r;
			float depthTemp = smallestDist2Light;
			fragColor = vec4(depthTemp, depthTemp, depthTemp, 1.0);
			*/
		}

		//vec3 lightDir = normalize(pointLight[0].pos - worldPos);
		//vec3 dColorOut = diffuseColor * (max(dot(normal, lightDir), 0.0) * diffuseIntensity);
		vec3 dColorOut = diffuseColor * dLightColor * diffuseIntensity * 1.0;

		//vec3 halfwayVector = normalize(lightDir + viewDir); 
		//vec3 sColorOut = specularColor * (pow(max(dot(normal, halfwayVector), 0.0), specularExp) * specularIntensity);
		vec3 sColorOut = specularColor * sLightColor * specularIntensity * 1.0;

		vec3 eColorOut = diffuseColor * emit;

		fragColor = vec4(dColorOut + sColorOut + eColorOut, 1.0);

		//float depthTemp = texture(scSpotLightShadowMap_0, colorTexUV).r;
		//fragColor = vec4(depthTemp, depthTemp, depthTemp, 1.0);
		/*
		vec3 lightVec = scSpotLight[0].pos - worldPos;
		float lightDist = length(lightVec); 
		float propatility = spotLightInShadow(0, worldPos, lightDist, scSpotLight[0].nearAndFarPlane.y);
		fragColor = vec4(propatility, propatility, propatility, 1.0);
		*/
	}  // end of if(length(normal) > 0.0)
	else
	{
		fragColor = vec4(diffuseColor, 1.0);
	}
	//fragColor = vec4(normal, 1.0);worldPos
	//fragColor = vec4(worldPos(depthT), 1.0);
	gl_FragDepth = depthT;
}