//#version 150
precision highp float;

#define DIRECTION_NONE 0
#define DIRECTION_HORIZONTAL 1
#define DIRECTION_VERTICAL 2


uniform sampler2D depthTexture;
uniform sampler2D moment2Texture;
uniform int operationMode; // samplingDirection;
uniform vec2 sampleSize;

in vec2 colorTexUV;

out vec4 fragDepth;
out vec4 fragMoment2;

float texture2DfromDepth(sampler2D yDepthTexture, vec2 yUV)
{
	vec4 texVal =  texture2D(yDepthTexture, yUV);
	if(operationMode == DIRECTION_VERTICAL)
	{
		return texVal.a;
	}
	else
	{
		return texVal.r;
	}
}

vec2 gauss(vec2 yUV, float yDD, int yDirection)
{
	//Initialisierungen
	float wights[7];
		wights[0] = 0.015625;
		wights[1] = 0.09375;
		wights[2] = 0.234375;
		wights[3] = 0.3125;
		wights[4] = 0.234375;
		wights[5] = 0.09375;
		wights[6] = 0.015625;
	
	vec2 offsets[7];
	if(yDirection == DIRECTION_HORIZONTAL)
	{
		offsets[0] = vec2(yDD*-3.0, 0.0);
		offsets[1] = vec2(yDD*-2.0, 0.0);
		offsets[2] = vec2(yDD*-1.0, 0.0);
		offsets[3] = vec2(0.0, 0.0);
		offsets[4] = vec2(yDD*1.0, 0.0);
		offsets[5] = vec2(yDD*2.0, 0.0);
		offsets[6] = vec2(yDD*3.0, 0.0);
	}
	else //  DIRECTION_VERTICAL
	{
		offsets[0] = vec2(0.0, yDD*-3.0);
		offsets[1] = vec2(0.0, yDD*-2.0);
		offsets[2] = vec2(0.0, yDD*-1.0);
		offsets[3] = vec2(0.0, 0.0);
		offsets[4] = vec2(0.0, yDD*1.0);
		offsets[5] = vec2(0.0, yDD*2.0);
		offsets[6] = vec2(0.0, yDD*3.0);
	}

	vec2 ret = vec2(0.0, 0.0);

	vec2 temp; 
	for(int i=0; i<7; i++)
	{
		temp.x = texture2DfromDepth(depthTexture, yUV + offsets[i]);
		temp.y = texture2D(moment2Texture, yUV + offsets[i]).a;
		ret += temp * wights[i];
	}
	
	return ret;
}


void main()
{
	//fragDepth = vec4(0.0, 0.0, 0.0, 0.0);
	//fragMoment2 = vec4(0.0, 0.0, 0.0, 0.0);

	vec2 moments;
	if(operationMode == DIRECTION_NONE)
	{
		moments.x = texture2DfromDepth(depthTexture, colorTexUV);
		moments.y = texture2D(moment2Texture, colorTexUV).a;
	}
	else if(operationMode == DIRECTION_HORIZONTAL)
	{
		moments = gauss(colorTexUV, sampleSize.x, operationMode);
	}
	else
	{
		moments = gauss(colorTexUV, sampleSize.y, operationMode);
	}

	//moments.x = texture2DfromDepth(depthTexture, colorTexUV);
	//moments.y = texture2D(moment2Texture, colorTexUV).a;
	//moments.x = 0.5;
	//moments.y = 0.8;

	fragDepth.a = moments.x;
	fragMoment2.a = moments.y;
	//fragMoment2 = vec4(1.0, 1.0, 1.0, 1.0);
}