

uniform vec2 nearAndFarPlane; // near and far plane for cm-cams
uniform vec4 wsLightPos; // world space light position
#ifdef GODRAYSPOT
	uniform int shadowMapResulution;
#endif

in vec4 wsPos; //wsPosFromGS;

#ifndef CUBEMAP
out vec4 fragDoment2;

float computeMoment2(float yDepth) // zweites moment fr die Variance Shadow Maps
{
	// Compute partial derivation of depth
	float dx = dFdx(yDepth);
	float dy = dFdy(yDepth);
	
	// Compute second moment over the pixel extents.
	float moment2 = yDepth*yDepth + 0.25*(dx*dx + dy*dy);

	return moment2;
	//return dFdx(0.5) + dFdy(0.5);
}
#endif

void main(void)
{
	#ifdef GODRAYSPOT
		vec2 sreenPos = vec2(gl_FragCoord) / shadowMapResulution * 2.0 - 1.0; // gl_FragCoord liefert werte von 0 bis shadowMapResulution, ich will sie aber zwischen -1 und 1;
		float radius = sqrt(pow(sreenPos.x, 2) + pow(sreenPos.y, 2));
		#define SIN5 0.08715574274765817 // um die 5 grad trashhold auszugleichen
		// etwas kleiner um das rauschen das durch die in HW gegossene partielle ableitung bei der secondMoment berechung fr VSM auftrit unsichtbar zu machen
		#define SIN4 0.06975647374412530 //sin(4*pi/180) 
		if(radius > (1.0 - SIN4)) // * nearAndFarPlane.x))
		{
			gl_FragDepth = 0.0;
		}
		else
		{
	#endif
			// calculate distance
			float wsDist = distance(wsPos, wsLightPos);

			// map value to [0;1] by dividing by far plane distance
			float wsDistNormalized = wsDist / nearAndFarPlane.y;
	
			// write modified depth
			gl_FragDepth = wsDistNormalized;

			#ifndef CUBEMAP
			// when using depth-only FBO, do NOT write to color!!!
			// but we write the second moment for the variance shadow maps to a 32 bit ALPHA tex
			fragDoment2.a = computeMoment2(wsDistNormalized);
			#endif
	#ifdef GODRAYSPOT
		}
	#endif
}