#version 150
precision highp float;

//const float middleGrey = 0.6f;
//const float maxLuminance = 16.0f;
const vec3 LUM_CONVERT = vec3(0.299f, 0.587f, 0.114f);

uniform sampler2D colorTexture;
uniform sampler2D glowTexture;
uniform sampler2D lumTexture;
uniform float middleGrey = 0.6f;
uniform float maxLuminance = 16.0f;
uniform float glowThreshold = 0.8f;

// Debug bools
uniform bool HDRshowColor = true;
uniform bool HDRshowGlow = false;
uniform bool HDRshowToneMappedColor = true;
uniform bool HDRshowToneMappedGlow = true;
//uniform bool showAvgLum;
uniform float HDRfade2Black = 1.0;

in vec2 colorTexUV;

out vec4 fragColor;



vec4 ToneMap(vec4 yColor)
{	
	// Get the calculated average luminance
	float lumAvg = texture2D(lumTexture, vec2(0.5f, 0.5f)).a;
	//vec4 lumAvgTexel = texture2D(lumTexture, vec2(0.5f, 0.5f));  
	//float lumAvg = (lumAvgTexel.r + lumAvgTexel.g + lumAvgTexel.b ) / 3;

	// Calculate the luminance of the current pixel
	float lumPixel = dot(yColor.rgb, LUM_CONVERT);    

	// Apply the modified operator (Eq. 4)
	float lumScaled = (lumPixel * middleGrey) / lumAvg;
	float lumCompressed = (lumScaled * (1 + (lumScaled / (maxLuminance * maxLuminance)))) / (1 + lumScaled);
	return lumCompressed * yColor;
}

vec4 GlowColor(vec4 yColor)
{
	vec4 color = ToneMap(yColor);

	color -= glowThreshold;
	return max(color, 0.0f);
}

void main()
{
	//fragColor vec4(1.0f, 1.0f, 1.0f, 1.0f);
	vec4 color = texture2D(colorTexture, colorTexUV);
	vec4 glowColor = texture2D(glowTexture, colorTexUV) / 2.0f;
	//fragColor = glowColor/3.0f; // / 3.0f weil die 3 texturen mit den ferschidenen auflsungen im glow einfach aufsumiert werden
	//fragColor = color + glowColor;
	
	
	// NORMALE ANZEIGE
	if(HDRshowToneMappedColor && HDRshowToneMappedGlow)
	{
		fragColor = ToneMap(color);
		//float tmpLum = texture2D(lumTexture, vec2(0.5f, 0.5f)).r;
		//fragColor = vec4(tmpLum, tmpLum, tmpLum, 1.0f);
		
		fragColor = fragColor + GlowColor(glowColor);
	}
	else
	{
		fragColor = vec4(0.0f, 0.0f, 0.0f, 1.0f);
		if(HDRshowToneMappedColor)
		{
			fragColor = ToneMap(color);
		}
		else if(HDRshowToneMappedGlow)
		{
			fragColor = GlowColor(glowColor);
		}
		
		if(HDRshowColor)
		{
			fragColor += color;
		}

		if(HDRshowGlow)
		{
			fragColor += glowColor;
		}
	}

	fragColor *= HDRfade2Black;
	fragColor.a = 1.0;
}